@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2021, The HSQL Development Group
+/* Copyright (c) 2001-2022, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -57,7 +57,6 @@ import org.hsqldb.rowio.RowOutputInterface;
 import org.hsqldb.server.HsqlSocketFactory;
 import org.hsqldb.types.BlobDataID;
 import org.hsqldb.types.ClobDataID;
-import org.hsqldb.types.TimestampData;
 
 /**
  * Base remote session proxy implementation. Uses instances of Result to
@@ -65,7 +64,7 @@ import org.hsqldb.types.TimestampData;
  * protocol.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.1
+ * @version 2.7.0
  * @since 1.7.2
  */
 public class ClientConnection implements SessionInterface, Cloneable {
@@ -130,7 +129,7 @@ public class ClientConnection implements SessionInterface, Cloneable {
     public ClientConnection(String host, int port, String path,
                             String database, boolean isTLS,
                             boolean isTLSWrapper, String user,
-                            String password, int timeZoneSeconds) {
+                            String password, TimeZone timeZone) {
 
         this.host         = host;
         this.port         = port;
@@ -138,14 +137,14 @@ public class ClientConnection implements SessionInterface, Cloneable {
         this.database     = database;
         this.isTLS        = isTLS;
         this.isTLSWrapper = isTLSWrapper;
-        this.zoneSeconds  = timeZoneSeconds;
-        this.zoneString   = TimeZone.getDefault().getID();
+        this.zoneSeconds  = timeZone.getOffset(System.currentTimeMillis()) / 1000;
+        this.zoneString   = timeZone.getID();
 
         initStructures();
         initConnection(host, port, isTLS);
 
         Result login = Result.newConnectionAttemptRequest(user, password,
-            database, zoneString, timeZoneSeconds);
+            database, zoneString, zoneSeconds);
         Result resultIn = execute(login);
 
         if (resultIn.isError()) {
@@ -250,7 +249,7 @@ public class ClientConnection implements SessionInterface, Cloneable {
 
             return read();
         } catch (Throwable e) {
-            throw Error.error(ErrorCode.X_08006, e.toString());
+            throw Error.error(e, ErrorCode.X_08006, e.toString());
         }
     }
 
@@ -267,7 +266,7 @@ public class ClientConnection implements SessionInterface, Cloneable {
 
             return (RowSetNavigatorClient) result.getNavigator();
         } catch (Throwable e) {
-            throw Error.error(ErrorCode.X_08006, e.toString());
+            throw Error.error(e, ErrorCode.X_08006, e.toString());
         }
     }
 
@@ -633,15 +632,6 @@ public class ClientConnection implements SessionInterface, Cloneable {
         return simpleDateFormatGMT;
     }
 
-    public TimestampData getCurrentDate() {
-
-        long currentMillis = System.currentTimeMillis();
-
-        currentMillis = HsqlDateTime.getNormalisedDate(currentMillis);
-
-        return new TimestampData(currentMillis / 1000);
-    }
-
     public int getZoneSeconds() {
         return zoneSeconds;
     }
