@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2021, The HSQL Development Group
+/* Copyright (c) 2001-2022, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,7 +59,7 @@ import org.hsqldb.types.Types;
  * timezone.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.6.0
+ * @version 2.7.0
  * @since 1.7.0
  */
 public class HsqlDateTime {
@@ -105,7 +105,7 @@ public class HsqlDateTime {
                 return d.getTime() / 1000;
             }
         } catch (Exception e) {
-            throw Error.error(ErrorCode.X_22007);
+            throw Error.error(ErrorCode.X_22007, e);
         }
     }
 
@@ -127,15 +127,16 @@ public class HsqlDateTime {
                 return d.getTime() / 1000;
             }
         } catch (Exception e) {
-            throw Error.error(ErrorCode.X_22007);
+            throw Error.error(ErrorCode.X_22007, e);
         }
     }
 
-    public static String getTimestampString(long seconds,
-                                          int nanos, int scale) {
+    public static String getTimestampString(long seconds, int nanos,
+            int scale) {
 
         synchronized (sdfts) {
             sysDate.setTime(seconds * 1000);
+
             String ts = sdfts.format(sysDate);
 
             if (scale > 0) {
@@ -190,26 +191,40 @@ public class HsqlDateTime {
         }
     }
 
-    public static long convertMillisFromCalendar(Calendar calendar,
-            long millis) {
-        return convertMillisFromCalendar(tempCalGMT, calendar, millis);
+    public static long convertMillisFromCalendar(Calendar sourceCalendar,
+            Calendar targetClendar, long millis) {
+
+        synchronized (targetClendar) {
+            synchronized (sourceCalendar) {
+                targetClendar.clear();
+                sourceCalendar.setTimeInMillis(millis);
+                targetClendar.set(sourceCalendar.get(Calendar.YEAR),
+                                  sourceCalendar.get(Calendar.MONTH),
+                                  sourceCalendar.get(Calendar.DAY_OF_MONTH),
+                                  sourceCalendar.get(Calendar.HOUR_OF_DAY),
+                                  sourceCalendar.get(Calendar.MINUTE),
+                                  sourceCalendar.get(Calendar.SECOND));
+
+                return targetClendar.getTimeInMillis();
+            }
+        }
     }
 
-    public static long convertMillisFromCalendar(Calendar clendarGMT,
-            Calendar calendar, long millis) {
+    public static long convertSecondsFromCalendar(Calendar sourceCalendar,
+            Calendar targetClendar, long seconds) {
 
-        synchronized (clendarGMT) {
-            synchronized (calendar) {
-                clendarGMT.clear();
-                calendar.setTimeInMillis(millis);
-                clendarGMT.set(calendar.get(Calendar.YEAR),
-                               calendar.get(Calendar.MONTH),
-                               calendar.get(Calendar.DAY_OF_MONTH),
-                               calendar.get(Calendar.HOUR_OF_DAY),
-                               calendar.get(Calendar.MINUTE),
-                               calendar.get(Calendar.SECOND));
+        synchronized (targetClendar) {
+            synchronized (sourceCalendar) {
+                targetClendar.clear();
+                sourceCalendar.setTimeInMillis(seconds * 1000);
+                targetClendar.set(sourceCalendar.get(Calendar.YEAR),
+                                  sourceCalendar.get(Calendar.MONTH),
+                                  sourceCalendar.get(Calendar.DAY_OF_MONTH),
+                                  sourceCalendar.get(Calendar.HOUR_OF_DAY),
+                                  sourceCalendar.get(Calendar.MINUTE),
+                                  sourceCalendar.get(Calendar.SECOND));
 
-                return clendarGMT.getTimeInMillis();
+                return targetClendar.getTimeInMillis() / 1000;
             }
         }
     }
@@ -282,21 +297,6 @@ public class HsqlDateTime {
         return calendar.getTimeZone().getOffset(millis);
     }
 
-    /**
-     * Returns the indicated part of the given millisecond date object.
-     * @param m the millisecond time value from which to extract the indicated part
-     * @param part an integer code corresponding to the desired date part
-     * @return the indicated part of the given <code>java.util.Date</code> object
-     */
-    public static int getDateTimePart(Calendar calendar, long m, int part) {
-
-        synchronized (calendar) {
-            calendar.setTimeInMillis(m);
-
-            return calendar.get(part);
-        }
-    }
-
     /**
      * truncates millisecond date object
      */
@@ -313,6 +313,7 @@ public class HsqlDateTime {
                     if (dayWeek == 1) {
                         dayWeek = 8;
                     }
+
                     calendar.add(Calendar.DAY_OF_YEAR, 2 - dayWeek);
                     resetToDate(calendar);
 
@@ -420,7 +421,7 @@ public class HsqlDateTime {
         }
     }
 
-    static void zeroFromPart(Calendar cal, int part) {
+    public static void zeroFromPart(Calendar cal, int part) {
 
         switch (part) {
 
@@ -533,7 +534,7 @@ public class HsqlDateTime {
 
             millis = format.parse(string).getTime();
         } catch (Exception e) {
-            throw Error.error(ErrorCode.X_22007, e.toString());
+            throw Error.error(e, ErrorCode.X_22007, e.toString());
         }
 
         if (matchIndex >= 0 && fraction) {
@@ -555,7 +556,7 @@ public class HsqlDateTime {
 
                 nanos *= (1000000 / factor);
             } catch (Exception e) {
-                throw Error.error(ErrorCode.X_22007, e.toString());
+                throw Error.error(e, ErrorCode.X_22007, e.toString());
             }
         }
 
@@ -570,7 +571,7 @@ public class HsqlDateTime {
         try {
             format.applyPattern(javaPattern);
         } catch (Exception e) {
-            throw Error.error(ErrorCode.X_22511);
+            throw Error.error(ErrorCode.X_22511, e);
         }
 
         String result     = format.format(date);
@@ -702,7 +703,7 @@ public class HsqlDateTime {
                     String s     = javaDateTokens[index];
 
                     // consecutive quoted tokens
-                    if (s.startsWith("\'") && s.endsWith("\'")) {
+                    if (s.startsWith("'") && s.endsWith("'")) {
                         if (limitQuotedToken == sb.length()) {
                             sb.setLength(sb.length() - 1);
 
