@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2021, The HSQL Development Group
+/* Copyright (c) 2001-2022, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,7 +56,7 @@ import org.hsqldb.types.Types;
  * Implementation of an SQL query expression
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.6.1
+ * @version 2.7.0
  * @since 1.9.0
  */
 public class QueryExpression implements RangeGroup {
@@ -755,7 +755,7 @@ public class QueryExpression implements RangeGroup {
                 break;
             }
 
-            if (round > 256) {
+            if (round > session.database.sqlMaxRecursive) {
                 throw Error.error(ErrorCode.X_22522);
             }
 
