@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2021, The HSQL Development Group
+/* Copyright (c) 2001-2022, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,7 +37,6 @@ import java.util.GregorianCalendar;
 import java.util.Locale;
 import java.util.Random;
 import java.util.TimeZone;
-import java.util.concurrent.atomic.AtomicInteger;
 
 import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.error.Error;
@@ -66,6 +65,7 @@ import org.hsqldb.rights.Grantee;
 import org.hsqldb.rights.User;
 import org.hsqldb.types.BlobDataID;
 import org.hsqldb.types.ClobDataID;
+import org.hsqldb.types.DateTimeType;
 import org.hsqldb.types.TimeData;
 import org.hsqldb.types.TimestampData;
 import org.hsqldb.types.Type;
@@ -75,7 +75,7 @@ import org.hsqldb.types.TypedComparator;
  * Implementation of SQL sessions.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.6.1
+ * @version 2.7.0
  * @since 1.7.0
  */
 public class Session implements SessionInterface {
@@ -95,11 +95,11 @@ public class Session implements SessionInterface {
     int isolationLevel        = SessionInterface.TX_READ_COMMITTED;
     boolean                 isReadOnlyIsolation;
     int                     actionIndex;
-    long                    actionStartTimestamp;
-    long                    actionTimestamp;
-    long                    statementStartTimestamp;
-    long                    transactionTimestamp;
-    long                    transactionEndTimestamp;
+    long                    actionStartSCN;
+    long                    actionSCN;
+    long                    statementStartSCN;
+    long                    transactionSCN;
+    long                    transactionEndSCN;
     final boolean           txConflictRollback;
     final boolean           txInterruptRollback;
     boolean                 isPreTransaction;
@@ -118,15 +118,15 @@ public class Session implements SessionInterface {
     TimeoutManager          timeoutManager;
 
     // current settings
-    final String       zoneString;
-    final int          sessionTimeZoneSeconds;
-    int                timeZoneSeconds;
-    boolean            isNetwork;
-    private int        sessionMaxRows;
-    int                sessionOptimization = 8;
-    private final long sessionId;
-    private boolean    ignoreCase;
-    private final long connectTime = System.currentTimeMillis();
+    final TimeZone              timeZone;
+    TimeZone                    currentTimeZone;
+    final String                zoneString;
+    boolean                     isNetwork;
+    private int                 sessionMaxRows;
+    int                         sessionOptimization = 8;
+    private final long          sessionId;
+    private boolean             ignoreCase;
+    private final TimestampData connectTimestamp;
 
     // internal connection
     private JDBCConnection intConnection;
@@ -156,49 +156,48 @@ public class Session implements SessionInterface {
 
     /**
      * Constructs a new Session object.
-     *
      * @param  db the database to which this represents a connection
      * @param  user the initial user
      * @param  autocommit the initial autocommit value
      * @param  readonly the initial readonly value
      * @param  id the session identifier, as known to the database
+     * @param  zone the TimeZone
      */
     Session(Database db, User user, boolean autocommit, boolean readonly,
-            long id, String zoneString, int timeZoneSeconds) {
+            long id, TimeZone zone) {
 
-        sessionId                   = id;
-        database                    = db;
-        this.user                   = user;
-        this.sessionUser            = user;
-        this.zoneString             = zoneString;
-        this.sessionTimeZoneSeconds = timeZoneSeconds;
-        this.timeZoneSeconds        = timeZoneSeconds;
-        rowActionList               = new HsqlArrayList(128, true);
-        waitedSessions              = new OrderedHashSet();
-        waitingSessions             = new OrderedHashSet();
-        tempSet                     = new OrderedHashSet();
-        actionSet                   = new OrderedHashSet();
-        isolationLevelDefault       = database.defaultIsolationLevel;
-        ignoreCase                  = database.sqlIgnoreCase;
-        isolationLevel              = isolationLevelDefault;
-        txConflictRollback          = database.txConflictRollback;
-        txInterruptRollback         = database.txInterruptRollback;
-        isReadOnlyDefault           = readonly;
+        this.database         = db;
+        this.sessionUser      = user;
+        this.user             = user;
+        this.sessionId        = id;
+        this.timeZone         = zone;
+        this.currentTimeZone  = zone;
+        this.zoneString       = zone.getID();
+        rowActionList         = new HsqlArrayList(128, true);
+        waitedSessions        = new OrderedHashSet();
+        waitingSessions       = new OrderedHashSet();
+        tempSet               = new OrderedHashSet();
+        actionSet             = new OrderedHashSet();
+        isolationLevelDefault = database.defaultIsolationLevel;
+        ignoreCase            = database.sqlIgnoreCase;
+        isolationLevel        = isolationLevelDefault;
+        txConflictRollback    = database.txConflictRollback;
+        txInterruptRollback   = database.txInterruptRollback;
+        isReadOnlyDefault     = readonly;
         isReadOnlyIsolation = isolationLevel
                               == SessionInterface.TX_READ_UNCOMMITTED;
         sessionContext              = new SessionContext(this);
-        sessionContext.isAutoCommit = autocommit ? Boolean.TRUE
-                                                 : Boolean.FALSE;
-        sessionContext.isReadOnly   = isReadOnlyDefault ? Boolean.TRUE
-                                                        : Boolean.FALSE;
+        sessionContext.isAutoCommit = autocommit;
+        sessionContext.isReadOnly   = isReadOnlyDefault;
         parser                      = new ParserCommand(this, new Scanner());
+        sessionData                 = new SessionData(database, this);
+        statementManager            = new StatementManager(this);
+        timeoutManager              = new TimeoutManager();
 
         setResultMemoryRowCount(database.getResultMaxMemoryRows());
         resetSchema();
 
-        sessionData      = new SessionData(database, this);
-        statementManager = new StatementManager(this);
-        timeoutManager   = new TimeoutManager();
+        connectTimestamp = getCurrentTimestamp();
     }
 
     void resetSchema() {
@@ -414,7 +413,7 @@ public class Session implements SessionInterface {
      */
     void checkReadWrite() {
 
-        if (sessionContext.isReadOnly.booleanValue() || isReadOnlyIsolation) {
+        if (sessionContext.isReadOnly || isReadOnlyIsolation) {
             throw Error.error(ErrorCode.X_25006);
         }
     }
@@ -431,8 +430,8 @@ public class Session implements SessionInterface {
         checkReadWrite();
     }
 
-    public long getActionTimestamp() {
-        return actionTimestamp;
+    public long getActionSCN() {
+        return actionSCN;
     }
 
     /**
@@ -453,7 +452,6 @@ public class Session implements SessionInterface {
             throw Error.error(ErrorCode.X_40502);
         }
 
-        getTransactionUTC();
         database.txManager.addDeleteAction(this, table, store, row,
                                            changedColumns);
     }
@@ -461,8 +459,6 @@ public class Session implements SessionInterface {
     void addInsertAction(Table table, PersistentStore store, Row row,
                          int[] changedColumns) {
 
-        getTransactionUTC();
-
 //        tempActionHistory.add("add insert to transaction " + actionTimestamp);
         database.txManager.addInsertAction(this, table, store, row,
                                            changedColumns);
@@ -496,14 +492,26 @@ public class Session implements SessionInterface {
             return;
         }
 
-        if (sessionContext.isAutoCommit.booleanValue() != autocommit) {
+        if (sessionContext.isAutoCommit != autocommit) {
             commit(false);
 
-            sessionContext.isAutoCommit = autocommit ? Boolean.TRUE
-                                                     : Boolean.FALSE;
+            sessionContext.isAutoCommit = autocommit;
         }
     }
 
+    public void setAutoCommitRows(int rows) {
+
+        if (isClosed) {
+            return;
+        }
+
+        if (sessionContext.depth > 0) {
+            return;
+        }
+
+        sessionContext.autoCommitRows = rows;
+    }
+
     public void beginAction(Statement cs) {
 
         actionIndex = rowActionList.size();
@@ -521,8 +529,7 @@ public class Session implements SessionInterface {
         sessionData.persistentStoreCollection.clearStatementTables();
 
         if (result.mode == ResultConstants.ERROR) {
-            sessionData.persistentStoreCollection.clearResultTables(
-                actionTimestamp);
+            sessionData.persistentStoreCollection.clearResultTables(actionSCN);
             database.txManager.rollbackAction(this);
         } else {
             sessionContext
@@ -639,8 +646,7 @@ public class Session implements SessionInterface {
         sessionData.closeAllTransactionNavigators();
 
         if (!chain) {
-            sessionContext.isReadOnly = isReadOnlyDefault ? Boolean.TRUE
-                                                          : Boolean.FALSE;
+            sessionContext.isReadOnly = isReadOnlyDefault;
 
             setIsolation(isolationLevelDefault);
         }
@@ -676,14 +682,14 @@ public class Session implements SessionInterface {
         statementManager.reset();
 
         sessionContext.lastIdentity = ValuePool.INTEGER_0;
-        sessionContext.isAutoCommit = Boolean.TRUE;
+        sessionContext.isAutoCommit = true;
 
         setResultMemoryRowCount(database.getResultMaxMemoryRows());
 
         user = sessionUser;
 
         resetSchema();
-        setZoneSeconds(sessionTimeZoneSeconds);
+        resetTimeZone();
 
         sessionMaxRows = 0;
         ignoreCase     = database.sqlIgnoreCase;
@@ -706,11 +712,11 @@ public class Session implements SessionInterface {
             sessionContext.savepointTimestamps.remove(index);
         }
 
-        actionTimestamp = database.txManager.getNextGlobalChangeTimestamp();
+        actionSCN = database.txManager.getNextSystemChangeNumber();
 
         sessionContext.savepoints.add(name,
                                       ValuePool.getInt(rowActionList.size()));
-        sessionContext.savepointTimestamps.addLast(actionTimestamp);
+        sessionContext.savepointTimestamps.addLast(actionSCN);
     }
 
     /**
@@ -780,7 +786,7 @@ public class Session implements SessionInterface {
     }
 
     public void setNoSQL() {
-        sessionContext.noSQL = Boolean.TRUE;
+        sessionContext.noSQL = true;
     }
 
     public void setIgnoreCase(boolean mode) {
@@ -806,8 +812,7 @@ public class Session implements SessionInterface {
             throw Error.error(ErrorCode.X_25001);
         }
 
-        sessionContext.isReadOnly = readonly ? Boolean.TRUE
-                                             : Boolean.FALSE;
+        sessionContext.isReadOnly = readonly;
     }
 
     public synchronized void setReadOnlyDefault(boolean readonly) {
@@ -819,8 +824,7 @@ public class Session implements SessionInterface {
         isReadOnlyDefault = readonly;
 
         if (!isInMidTransaction()) {
-            sessionContext.isReadOnly = isReadOnlyDefault ? Boolean.TRUE
-                                                          : Boolean.FALSE;
+            sessionContext.isReadOnly = isReadOnlyDefault;
         }
     }
 
@@ -830,7 +834,7 @@ public class Session implements SessionInterface {
      * @return the current value
      */
     public boolean isReadOnly() {
-        return sessionContext.isReadOnly.booleanValue() || isReadOnlyIsolation;
+        return sessionContext.isReadOnly || isReadOnlyIsolation;
     }
 
     public synchronized boolean isReadOnlyDefault() {
@@ -843,7 +847,7 @@ public class Session implements SessionInterface {
      * @return the current value
      */
     public synchronized boolean isAutoCommit() {
-        return sessionContext.isAutoCommit.booleanValue();
+        return sessionContext.isAutoCommit;
     }
 
     public synchronized int getStreamBlockSize() {
@@ -907,8 +911,8 @@ public class Session implements SessionInterface {
      *
      * @return the value
      */
-    public long getConnectTime() {
-        return connectTime;
+    public TimestampData getConnectTimestamp() {
+        return connectTimestamp;
     }
 
     /**
@@ -920,8 +924,8 @@ public class Session implements SessionInterface {
         return rowActionList.size();
     }
 
-    public long getTransactionTimestamp() {
-        return transactionTimestamp;
+    public long getTransactionSCN() {
+        return transactionSCN;
     }
 
     public Statement compileStatement(String sql, int props) {
@@ -1229,7 +1233,7 @@ public class Session implements SessionInterface {
 
             if (i > 0) {
                 if (cs.getCompileTimestamp()
-                        > database.txManager.getGlobalChangeTimestamp()) {
+                        > database.txManager.getSystemChangeNumber()) {
                     recompile = true;
                 }
 
@@ -1280,8 +1284,7 @@ public class Session implements SessionInterface {
         }
 
         if (sessionContext.depth > 0) {
-            if (sessionContext.noSQL.booleanValue()
-                    || cs.isAutoCommitStatement()) {
+            if (sessionContext.noSQL || cs.isAutoCommitStatement()) {
                 return Result.newErrorResult(Error.error(ErrorCode.X_46000));
             }
         }
@@ -1301,14 +1304,12 @@ public class Session implements SessionInterface {
         }
 
         sessionContext.currentStatement = cs;
-        statementStartTimestamp =
-            database.txManager.getGlobalChangeTimestamp();
+        statementStartSCN = database.txManager.getSystemChangeNumber();
 
         boolean isTX = cs.isTransactionStatement();
 
         if (!isTX) {
-            actionTimestamp =
-                database.txManager.getNextGlobalChangeTimestamp();
+            actionSCN = database.txManager.getNextSystemChangeNumber();
 
             sessionContext.setDynamicArguments(pvals);
 
@@ -1430,20 +1431,31 @@ public class Session implements SessionInterface {
             return handleAbortTransaction();
         }
 
-        if (sessionContext.depth == 0
-                && (sessionContext.isAutoCommit.booleanValue()
-                    || cs.isAutoCommitStatement())) {
-            try {
-                if (r.mode == ResultConstants.ERROR) {
-                    rollbackNoCheck(false);
-                } else {
-                    commit(false);
-                }
-            } catch (Exception e) {
-                sessionContext.currentStatement = null;
+        if (sessionContext.depth == 0) {
+            if (sessionContext.isAutoCommit || cs.isAutoCommitStatement()) {
+                try {
+                    if (r.mode == ResultConstants.ERROR) {
+                        rollbackNoCheck(false);
+                    } else {
+                        commit(false);
+                    }
+                } catch (Exception e) {
+                    sessionContext.currentStatement = null;
 
-                return Result.newErrorResult(Error.error(ErrorCode.X_40001,
-                        e));
+                    return Result.newErrorResult(Error.error(ErrorCode.X_40001,
+                            e));
+                }
+            } else if (sessionContext.autoCommitRows > 0) {
+                if (rowActionList.size() > sessionContext.autoCommitRows) {
+                    try {
+                        commit(false);
+                    } catch (Exception e) {
+                        sessionContext.currentStatement = null;
+
+                        return Result.newErrorResult(
+                            Error.error(ErrorCode.X_40001, e));
+                    }
+                }
             }
         }
 
@@ -1632,38 +1644,57 @@ public class Session implements SessionInterface {
     }
 
 // session DATETIME functions
-    long                  currentTimestampSCN;
-    long                  currentMillis;
+    long                  currentTimestampSCN = -1;    // initialise to invalid val
+    private TimestampData transactionUTC;
+    boolean               transactionUTCSet;
     private TimestampData currentDate;
     private TimestampData currentTimestamp;
     private TimestampData localTimestamp;
-    private TimestampData transactionUTC;
-    boolean               transactionUTCSet;
     private TimeData      currentTime;
     private TimeData      localTime;
 
     /**
-     * Returns the current date, unchanged for the duration of the current
+     * Returns the current date/time, unchanged for the duration of the current
      * execution unit (statement).<p>
      *
-     * SQL standards require that CURRENT_DATE, CURRENT_TIME and
-     * CURRENT_TIMESTAMP are all evaluated at the same point of
-     * time in the duration of each SQL statement, no matter how long the
-     * SQL statement takes to complete.<p>
+     * SQL standards require that CURRENT_DATE, CURRENT_TIME,
+     * CURRENT_TIMESTAMP, LOCALTIME, and LOCALTIMESTAMP are all evaluated at
+     * the same point of time in the duration of each SQL statement, no matter
+     * how long the SQL statement takes to complete.<p>
      *
      * When this method or a corresponding method for CURRENT_TIME or
      * CURRENT_TIMESTAMP is first called in the scope of a system change
-     * number, currentMillis is set to the current system time. All further
-     * CURRENT_XXXX calls in this scope will use this millisecond value.
+     * number, currentTimestamp is set to the current system time. All further
+     * CURRENT_XXXX and LOCALXXXX calls in this scope will use this base point
+     * of time.
+     *
      * (fredt@users)
      */
+    synchronized TimestampData getCurrentTimestamp() {
+
+        resetCurrentTimestamp();
+
+        return currentTimestamp;
+    }
+
+    synchronized TimestampData getLocalTimestamp() {
+
+        resetCurrentTimestamp();
+
+        if (localTimestamp == null) {
+            localTimestamp =
+                DateTimeType.toLocalTimestampValue(currentTimestamp);
+        }
+
+        return localTimestamp;
+    }
+
     public synchronized TimestampData getCurrentDate() {
 
         resetCurrentTimestamp();
 
         if (currentDate == null) {
-            currentDate = (TimestampData) Type.SQL_DATE.getValue(this,
-                    currentMillis / 1000, 0, getZoneSeconds());
+            currentDate = DateTimeType.toCurrentDateValue(currentTimestamp);
         }
 
         return currentDate;
@@ -1673,110 +1704,112 @@ public class Session implements SessionInterface {
      * Returns the current time, unchanged for the duration of the current
      * execution unit (statement)
      */
-    synchronized TimeData getCurrentTime(boolean withZone) {
+    synchronized TimeData getCurrentTime() {
 
         resetCurrentTimestamp();
 
-        if (withZone) {
-            if (currentTime == null) {
-                int seconds =
-                    (int) (HsqlDateTime.getNormalisedTime(
-                        getCalendarGMT(), currentMillis)) / 1000;
-                int nanos = (int) (currentMillis % 1000) * 1000000;
-
-                currentTime = new TimeData(seconds, nanos, getZoneSeconds());
-            }
-
-            return currentTime;
-        } else {
-            if (localTime == null) {
-                int seconds =
-                    (int) (HsqlDateTime.getNormalisedTime(
-                        getCalendarGMT(),
-                        currentMillis + getZoneSeconds() * 1000L)) / 1000;
-                int nanos = (int) (currentMillis % 1000) * 1000000;
-
-                localTime = new TimeData(seconds, nanos, 0);
-            }
-
-            return localTime;
+        if (currentTime == null) {
+            currentTime =
+                DateTimeType.toCurrentTimeWithZoneValue(currentTimestamp);
         }
+
+        return currentTime;
     }
 
-    /**
-     * Returns the current timestamp, unchanged for the duration of the current
-     * execution unit (statement)
-     */
-    synchronized TimestampData getCurrentTimestamp(boolean withZone) {
+    synchronized TimeData getLocalTime() {
 
         resetCurrentTimestamp();
 
-        if (withZone) {
-            if (currentTimestamp == null) {
-                int nanos = (int) (currentMillis % 1000) * 1000000;
-
-                currentTimestamp = new TimestampData((currentMillis / 1000),
-                                                     nanos, getZoneSeconds());
-            }
-
-            return currentTimestamp;
-        } else {
-            if (localTimestamp == null) {
-                int nanos = (int) (currentMillis % 1000) * 1000000;
-
-                localTimestamp = new TimestampData(currentMillis / 1000
-                                                   + getZoneSeconds(), nanos,
-                                                       0);
-            }
-
-            return localTimestamp;
+        if (localTime == null) {
+            localTime = DateTimeType.toCurrentTimeValue(currentTimestamp);
         }
+
+        return localTime;
     }
 
-    static TimestampData getSystemTimestamp(boolean withZone, boolean utc) {
+    private void resetCurrentTimestamp() {
 
-        long millis  = System.currentTimeMillis();
-        long seconds = millis / 1000;
-        int  nanos   = (int) (millis % 1000) * 1000000;
-        int  offset  = 0;
-
-        if (!utc) {
-            TimeZone zone = TimeZone.getDefault();
-
-            offset = zone.getOffset(millis) / 1000;
-
-            if (!withZone) {
-                seconds += offset;
-                offset  = 0;
-            }
+        if (currentTimestampSCN != actionSCN) {
+            currentTimestampSCN = actionSCN;
+            currentTimestamp =
+                DateTimeType.newCurrentTimestamp(currentTimeZone);
+            currentDate    = null;
+            localTimestamp = null;
+            currentTime    = null;
+            localTime      = null;
         }
-
-        return new TimestampData(seconds, nanos, offset);
     }
 
     TimestampData getTransactionUTC() {
 
         if (!transactionUTCSet) {
-            transactionUTC    = getSystemTimestamp(true, true);
+            transactionUTC    = DateTimeType.newSystemTimestampUTC();
             transactionUTCSet = true;
         }
 
         return transactionUTC;
     }
 
-    private void resetCurrentTimestamp() {
+    // session calendars
+    private Calendar calendar;
+    private Calendar calendarGMT;
 
-        if (currentTimestampSCN != actionTimestamp) {
-            currentTimestampSCN = actionTimestamp;
-            currentMillis       = System.currentTimeMillis();
-            currentDate         = null;
-            currentTimestamp    = null;
-            localTimestamp      = null;
-            currentTime         = null;
-            localTime           = null;
+    public int getZoneSeconds() {
+        return currentTimestamp.getZone();
+    }
+
+    public void resetTimeZone() {
+
+        currentTimeZone = timeZone;
+
+        if (calendar != null) {
+            calendar.setTimeZone(currentTimeZone);
         }
     }
 
+    public void setTimeZone(TimeZone zone) {
+
+        currentTimeZone = zone;
+
+        if (calendar != null) {
+            calendar.setTimeZone(currentTimeZone);
+        }
+
+        resetCurrentTimestamp();
+    }
+
+    public Calendar getCalendar() {
+
+        if (calendar == null) {
+            calendar = new GregorianCalendar(currentTimeZone);
+        }
+
+        return calendar;
+    }
+
+    public Calendar getCalendarGMT() {
+
+        if (calendarGMT == null) {
+            calendarGMT = new GregorianCalendar(TimeZone.getTimeZone("GMT"));
+
+            calendarGMT.setFirstDayOfWeek(Calendar.MONDAY);
+            calendarGMT.setMinimalDaysInFirstWeek(4);
+        }
+
+        return calendarGMT;
+    }
+
+    public SimpleDateFormat getSimpleDateFormatGMT() {
+
+        if (simpleDateFormatGMT == null) {
+            simpleDateFormatGMT = new SimpleDateFormat("MMMM", Locale.ENGLISH);
+
+            simpleDateFormatGMT.setCalendar(getCalendarGMT());
+        }
+
+        return simpleDateFormatGMT;
+    }
+
     private Result getAttributesResult(int id) {
 
         Result   r    = Result.newSessionAttributesResult();
@@ -1870,8 +1903,7 @@ public class Session implements SessionInterface {
                 return sessionContext.isAutoCommit;
 
             case SessionInterface.INFO_CONNECTION_READONLY :
-                return isReadOnlyDefault ? Boolean.TRUE
-                                         : Boolean.FALSE;
+                return isReadOnlyDefault;
 
             case SessionInterface.INFO_CATALOG :
                 return database.getCatalogName().name;
@@ -2005,7 +2037,7 @@ public class Session implements SessionInterface {
         if (result.getType() == ResultConstants.SQLCANCEL) {
             if (result.getSessionRandomID() == randomId) {
                 database.txManager.resetSession(
-                    this, this, statementStartTimestamp,
+                    this, this, statementStartSCN,
                     TransactionManager.resetSessionStatement);
             }
         }
@@ -2128,56 +2160,6 @@ public class Session implements SessionInterface {
         }
     }
 
-    // session zone
-    private Calendar calendar;
-    private Calendar calendarGMT;
-
-    public int getZoneSeconds() {
-        return timeZoneSeconds;
-    }
-
-    public void setZoneSeconds(int seconds) {
-        timeZoneSeconds = seconds;
-    }
-
-    public Calendar getCalendar() {
-
-        if (calendar == null) {
-            if (zoneString == null) {
-                calendar = new GregorianCalendar();
-            } else {
-                TimeZone zone = TimeZone.getTimeZone(zoneString);
-
-                calendar = new GregorianCalendar(zone);
-            }
-        }
-
-        return calendar;
-    }
-
-    public Calendar getCalendarGMT() {
-
-        if (calendarGMT == null) {
-            calendarGMT = new GregorianCalendar(TimeZone.getTimeZone("GMT"));
-
-            calendarGMT.setFirstDayOfWeek(Calendar.MONDAY);
-            calendarGMT.setMinimalDaysInFirstWeek(4);
-        }
-
-        return calendarGMT;
-    }
-
-    public SimpleDateFormat getSimpleDateFormatGMT() {
-
-        if (simpleDateFormatGMT == null) {
-            simpleDateFormatGMT = new SimpleDateFormat("MMMM", Locale.ENGLISH);
-
-            simpleDateFormatGMT.setCalendar(getCalendarGMT());
-        }
-
-        return simpleDateFormatGMT;
-    }
-
     // services
     TypedComparator  typedComparator = new TypedComparator(this);
     Scanner          secondaryScanner;
@@ -2326,11 +2308,11 @@ public class Session implements SessionInterface {
 
     // timeouts
     class TimeoutManager {
-
         volatile int  currentTimeout;
         volatile long checkTimestampSCN;
+        volatile long checkTimestamp;
 
-        void startTimeout(int timeout) {
+        synchronized void startTimeout(int timeout) {
 
             if (timeout == 0) {
                 return;
@@ -2338,31 +2320,30 @@ public class Session implements SessionInterface {
 
             boolean add = checkTimestampSCN == 0;
 
-            currentTimeout    = timeout + 1;
-            checkTimestampSCN = statementStartTimestamp;
+            currentTimeout    = timeout * 1000;
+            checkTimestampSCN = statementStartSCN;
+            checkTimestamp    = System.currentTimeMillis();
 
             if (add) {
                 database.timeoutRunner.addSession(this);
             }
         }
 
-        void endTimeout() {
+        synchronized void endTimeout() {
             currentTimeout = 0;
         }
 
-        public boolean checkTimeout() {
+        synchronized public boolean checkTimeout(long systemMillis) {
 
             if (currentTimeout == 0) {
                 return false;
             }
 
-            if (checkTimestampSCN != statementStartTimestamp) {
+            if (checkTimestampSCN != statementStartSCN) {
                 return false;
             }
 
-            currentTimeout--;
-
-            if (currentTimeout <= 0) {
+            if (checkTimestamp + currentTimeout < systemMillis) {
                 currentTimeout = 0;
 
                 database.txManager.resetSession(
@@ -2375,7 +2356,7 @@ public class Session implements SessionInterface {
             return false;
         }
 
-        public boolean isClosed() {
+        synchronized public boolean isClosed() {
             return Session.this.isClosed;
         }
     }
