@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2018, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -70,62 +70,63 @@ import junit.framework.TestResult;
  *    </servlet-mapping>
  * }</code>
  * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
- * @version 1.7.2
+ * @version 2.6.0
  * @since 1.7.2
  */
 public abstract class TestBase extends TestCase {
 
-    String  dbPath = "mem:test;sql.enforce_strict_size=true;sql.restrict_exec=true;hsqldb.tx=mvcc";
-    String  serverProps;
-    String  url;
-    String  user     = "sa";
-    String  password = "";
-    Server  server;
-    boolean isNetwork = true;
-    boolean isHTTP    = false;    // Set false to test HSQL protocol, true to test HTTP, in which case you can use isUseTestServlet to target either HSQL's webserver, or the Servlet server-mode
-    boolean isServlet = false;
+    static TestConnectionSettings settings =
+        new TestConnectionSettings.TestConnectionSettingsMem();
+    String        serverProps;
+    final String  url;
+    final String  user     = "sa";
+    final String  password = "";
+    Server        server;
+    final boolean isNetwork;
+    final boolean isHTTP;    // Set false to test HSQL protocol, true to test HTTP, in which case you can use isUseTestServlet to target either HSQL's webserver, or the Servlet server-mode
+    final boolean isServlet;
 
     public TestBase(String name) {
-        super(name);
-    }
-
-    public TestBase(String name, boolean isNetwork,
-                    boolean isHTTP) {
 
         super(name);
 
-        this.isNetwork = isNetwork;
-        this.isHTTP    = isHTTP;
-    }
+        this.url = settings.url();
 
-    public TestBase(String name, String url, boolean isNetwork,
-                    boolean isHTTP) {
+        String type = settings.connType();
 
-        super(name);
-
-        if (url != null) {
-            this.url = url;
+        if ("hsql:".equalsIgnoreCase(type)) {
+            this.isNetwork = true;
+            this.isHTTP    = false;
+            this.isServlet = false;
+        } else if ("http:".equalsIgnoreCase(type)) {
+            this.isNetwork = true;
+            this.isHTTP    = true;
+            this.isServlet = settings.isServlet();
+        } else {
+            this.isNetwork = false;
+            this.isHTTP    = false;
+            this.isServlet = false;
         }
+    }
 
-        this.isNetwork = isNetwork;
-        this.isHTTP    = isHTTP;
+    public TestBase(String name, String userUrl) {
+
+        super(name);
+
+        this.url       = userUrl;
+        this.isNetwork = false;
+        this.isHTTP    = false;
+        this.isServlet = false;
     }
 
     protected void setUp() throws Exception {
 
+        System.out.println(
+            "------------------------------------------------------------------------");
+        System.out.println("***   " + this.getClass().getName() + "  "
+                           + getName() + "   ******");
+
         if (isNetwork) {
-
-            //  change the url to reflect your preferred db location and name
-            if (url == null) {
-                if (isServlet) {
-                    url = "jdbc:hsqldb:http://localhost:8080/HSQLwebApp/test";
-                } else if (isHTTP) {
-                    url = "jdbc:hsqldb:http://localhost:8085/test";
-                } else {
-                    url = "jdbc:hsqldb:hsql://localhost/test";
-                }
-            }
-
             if (!isServlet) {
                 server = isHTTP ? new WebServer()
                                 : new Server();
@@ -135,15 +136,11 @@ public abstract class TestBase extends TestCase {
                 }
 
                 server.setDatabaseName(0, "test");
-                server.setDatabasePath(0, dbPath);
+                server.setDatabasePath(0, settings.dbPath());
                 server.setLogWriter(null);
                 server.setErrWriter(null);
                 server.start();
             }
-        } else {
-            if (url == null) {
-                url = "jdbc:hsqldb:" + dbPath;
-            }
         }
 
         try {
